{% load i18n %}
<a class="headerlink {% block urlhash_cls %}js-comment-urlhash{% endblock urlhash_cls %}" href="#{{ urlhash }}" title="{% trans 'Permalink to this comment' %}">
    {% block urlhash_icon %}
    ¶
    {% endblock urlhash_icon %}
</a>
