<table role="presentation" border="0" cellpadding="0" cellspacing="0">
    <tr>
        <td class="content-block">
            <span class="apple-link">Company Inc, 3 Abbey Road, San Francisco CA 94102</span>
            <br> Don't like these emails? <a href="http://i.imgur.com/CScmqnj.gif">Unsubscribe</a>.
        </td>
    </tr>
    <tr>
        <td class="content-block powered-by">Powered by <a href="http://htmlemail.io">HTMLemail</a>.
        </td>
    </tr>
</table>