{% load static %}
<link rel="stylesheet" href="{% static 'css/comment.css' %}">
{% if is_translation_allowed %}
<script type="text/javascript" src="{% url 'comment:javascript-catalog' %}"></script>
{% else %}
<script>
    /* This will allow django to detect strings for javascript translations */
    const gettext = str => str;
</script>
{% endif %}
<script src="{% static 'js/comment.js' %}"></script>
