{% load i18n %}
{% load comment_tags %}
<span title="{% if user|has_followed:comment %}{% trans 'Unfollow this thread' %}{% else %}{% trans 'Follow this thread' %}{% endif %}">
{% block follow_img_icon %}
<svg xmlns="http://www.w3.org/2000/svg" width="15" height="15" viewBox="0 0 24 24" fill="none"
     stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"
     class="comment-follow-icon {% if user|has_followed:model_object %}user-has-followed{% endif %}">
    <path d="M22 11.08V12a10 10 0 1 1-5.93-9.14"></path>
    <polyline points="22 4 12 14.01 9 11.01"></polyline>
</svg>
{% endblock follow_img_icon %}
</span>
