{% load comment_tags %}
{% load i18n %}
<button title="{% if comment.has_rejected_state %}{% trans "Flag rejected" %}{% else %}{% trans "Reject the flag" %}{% endif %}" class="js-flag-reject {% block reject_btn_cls %}btn btn-link{% endblock reject_btn_cls %}" data-state="{{ comment.flag.REJECTED }}" data-url="{% url 'comment:flag-change-state' pk=comment.id %}">

{% block reject_img_icon %}
<svg class="{% if comment.has_rejected_state %}flag-rejected{% endif %}"
     xmlns="http://www.w3.org/2000/svg" width="18" height="18" viewBox="0 0 24 24"
     fill="none" stroke="{% block reject_icon_color %}#E74C3C{% endblock reject_icon_color %}"
     stroke-width="1.7" stroke-linecap="round" stroke-linejoin="round">
     <path d="m19.216 15.194c0.31035 0.06374 0.55087-0.12433 0.77177-0.18337 4e-3 -0.0037 7e-3 -0.0065 9e-3 -0.0084 6e-3 -3.9981 3e-3 -8.0025 3e-3 -12.003 0 0-1 1-4 1s-5-2-8-2-4 1-4 1v12"></path>
     <line x1="4" x2="4" y1="22" y2="15"></line>
     <g transform="matrix(1.0063 0 0 1.0063 50.624 -.78936)">
      <g transform="matrix(1.1808 0 0 1.1808 55.855 .43703)">
       <path d="m-83.031 14.926 6.7248-6.7248"></path>
       <path d="m-76.306 14.926-6.7248-6.7248"></path>
      </g>
     </g>
</svg>
{% endblock reject_img_icon %}
</button>
