{% load i18n %}
{% load comment_tags %}
<div id="followModal" class="comment-modal">
    <div class="comment-modal-dialog" role="document">
        <div class="comment-modal-content">
            <div class="modal-header">
                <h5 class="modal-title">{% block title %}{% trans "Please insert your email to follow this thread" %}{% endblock title %}</h5>
                <button class="modal-close-btn" aria-label="{% trans "Close" %}">
                    <span aria-hidden="true">&times;</span>
                </button>
            </div>
            <div class="modal-body">
                <form class="js-comment-follow-form">
                {% block email_input %}
                    <div class="row">
                        <div class="col-3">
                            <label for="email">Email:</label>
                        </div>
                        <div class="col-9">

                            <input id="email" class="form-control mr-2 w-100" type="email" name="email" required>
                            <div class="error text-danger small mt-1"></div>
                        </div>
                    </div>
                {% endblock email_input %}
                    <br>
                    <div class="follow-modal-footer modal-footer">
                        {% trans "Follow" as follow %}
                        <button type="submit"
                                title="{{ follow }}"
                                class="btn btn-sm btn-success follow-modal-submit {% block follow_btn_extra_cls %}{% endblock follow_btn_extra_cls %}"
                        data-url="{% url 'comment:toggle-subscription' %}">{{ follow }}</button>
                    </div>
                </form>
            </div>
        </div>
    </div>
</div>

